extern "C" {
#include "mgraph.h"
}

/*
 * edge descriptor f =   00 | e | y     | x
 *                            3  YBITS   XBITS
 *
 * e is 0..5.  The edge is edge e out of vertex (x,y).
 *
 * BGL expects an undirected graph's edges to have two descriptors
 * each, one in each direction.
 */

/*
 * We use BGL's implementation of Dijkstra's single source shortest
 * paths.  We really want all pairs shortest paths, so Johnson All
 * Pairs Shortest Paths would seem sensible.  But actually Johnson's
 * algorithm is just a wrapper around Dijkstra's; the extra
 * functionality is just to deal with -ve edge weights, which we don't
 * have.  So we can use Dijkstra directly and save some cpu (and some
 * code: we don't have to supply all of the machinery needed for
 * Johnson's invocation of Bellman-Ford).  The overall time cost is
 * O(VE log V); I think the space used is O(E).
 */

#define VMASK (YMASK|XMASK)
#define ESHIFT (YBITS|XBITS)

namespace boost {
  // We make Layout a model of various BGL Graph concepts.
  // This mainly means that graph_traits<Layout> has lots of stuff.

  // First, some definitions used later:
  
  struct layout_graph_traversal_category : 
    public virtual incidence_graph_tag,
    public virtual vertex_list_graph_tag,
    public virtual edge_list_graph_tag { };
  
  struct OutEdgeIncrable {
    int f;
    OutEdgeIncrable& operator++() { f += 1<<ESHIFT; return self; }
    OutEdgeIncrable(int v, int e) : f(v | (e << ESHIFT)) { }
  };
 
  struct graph_traits<Layout> {

    // Concept Graph:
  
    typedef int vertex_descriptor; /* vertex number, -1 => none */
    typedef int edge_descriptor; /* see above */
    typedef undirected_tag directed_category;
    typedef disallow_parallel_ege edge_parallel_category;
    typedef layout_graph_traversal_category traversal_category;
    inline int null_vertex() { return -1; }

    // Concept IncidenceGraph:
    
    typedef counting_iterator<OutEdgeIncrable,
      forward_iterator_tag> out_edge_iterator;
    typedef int degree_size_type;
    
    inline int source(int f, const Layout&) { return f&VMASK; }
    inline int target(int f, const Layout&) { return EDGE_END2(f&VMASK, f>>ESHIFT); }
    inline std::pair<out_edge_iterator,out_edge_iterator>
    out_edges(int v, const Layout&) {
      return std::make_pair(out_edge_iterator(OutEdgeIncrable(v, VE_MIN(v))),
			    out_edge_iterator(OutEdgeIncrable(v, VE_MAX(v))));
    }
    out_degree(int v, const Layout&) { return VE_MAX(v) - VE_MIN(v); }

    // Concept VertexListGraph:
    typedef counting_iterator<int> vertex_iterator;
    typedef unsigned vertices_size_type;
    inline std::pair<vertex_iterator,vertex_iterator>
    vertices(const Layout&) {
      return std::make_pair(vertex_iterator(0), vertex_iterator(N));
    }
    inline unsigned num_vertices(const Layout&) { return N; }

}

struct VertexIndexMap;

namespace boost {
  struct property_traits<VertexIndexMap> {
    // Concept Readable Property Map:
    typedef int value_type, reference, key_type;
    category 
class Boost
  }};

void single_source_shortest_paths(int v1,
				  const double edge_weights[/*f*/],
				  ) {
  boost::dijkstra_shortest_paths
    (g, v1,
     weight_map(edge_weights).
     vertex_index_map(identity_property_map()).
     
    
void all_pairs_shortest_paths(const Layout *g) {
  
  FOR_VERTEX(v1) {
       
 0);
    
			    /* weight_map(). ? */
			    /* 

			    
			    predecessor_map().
			    distance_map()



